%%%%%%%%%%%%%%%%%%%%%%%% A Model with non-tradable goods and mining sector
%%%%%%%%% this program perfoms deterministic simulation including multiple shocks under covid-19 scenario

% Author: Gunbileg Ganbayar

close all;
warning off
%%
%%%%%%%%%%%%%%%%%%%%%%%Endogenous Variables %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
var
c               % consumption
rk              % rental rate of capital
w               % wage
h               % total hours
hT              % T hours
hN              % N hours
yT              % T output
yN              % N output
yO				% Mining output 
k               % total capital
kT              % T capital
kN              % N capital
q               % Tobin Q
i               % investement
lambda          % marginal utility of consumption
d               % debt
p               % CPI (or real exchange rate)
pN              % N price
pO				% Mining price 
tb              % trade balance
gdp             % gross domestic product
g               % government spending     
aT              % T TFP
aN              % N TFP
rz              % foreign rate
ac 				% preference shock
Z				% auxiliary variable for checking market clearing

// observed variables
NX_GDP GDP_Growth G_Growth M_GDP_Growth M_Price_Growth Cons_Growth Foreign_Rate

% log variables to have IRFs in percentage deviations from the ss
clog
wlog 
hlog
hTlog
hNlog
yNlog
yTlog
gdplog
klog
kNlog
kTlog
ilog
pNlog
plog
dy    
dlog
;
%%
%%%%%%%%%%%%%%%%%%%%%%%Exogenous Variables%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
varexo 
vaT  % tradable productivity shock
vaN  % non-tradable productivity shock
vg   % public spending shock
vp   % foreign monetary policy shock

vpO  % commodity price shock
vyO	 % mining production shock
vc 	 % preference shock
;  
    
%%
%%%%%%%%%%%%%%%%%%%%%%%Parameters%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

parameters

beta 
alphaT
alphaN
delta
sigma
phi 
kappaL
eta
gamma
kNss
kTss
hNss
hTss
aTss
rzss
dss
gss
pNss
pss
kappaD
kappaI
yOss 
O 
tO
rhoaT
rhoaN
rhog
rhop 
rhoyO 
rhopO 
rhoc
sdT
sdN
sdg
sdp
sdyO
sdpO
sdc
;
%%%estimated and calibrated parameters value%%
%%%the parameters values not calibrated are from console_mining running.
beta=0.99;
alphaT=0.4;
alphaN=0.33;
delta=0.025;
sigma=2;
phi=1;
kappaL=7.6389;
eta=1.44;
gamma=0.58;
kNss=5.7944;
kTss=8.2775;
hNss=0.1622;
hTss=0.1712;
aTss=1;
rzss=1.0101;
dss=7.2812;
gss=0.2128;
pNss=1.2983;
pss=1.1118;
kappaD=0.0015;
kappaI=2.48;
O=0.2;
tO=0.2;
yOss=0.3641;
%%% posterior mean of bayesian estimation
%% persistence
rhoaT=0.7589;
rhoaN=0.5825;
rhog=0.8681;
rhop=0.9462;
rhoyO=0.9471;
rhopO=0.9703;
rhoc=0.795;
%% standard errors
sdT=0.1296;
sdN=0.0048;
sdg=0.1606;
sdp=0.0015;
sdyO=0.0959;
sdpO=0.2567;
sdc=0.3929;
 
%%
%%%%%%%%%%%%%%%%%%%%%%%Non-Linear Model%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

model;
% Households
lambda=ac*(c-kappaL/(1+phi)*h^(1+phi))^-sigma;  
1=beta*lambda(1)/lambda*rz*p/p(1)+kappaD*(d-dss);
1=beta*lambda(1)/lambda*(rk(1)+(1-delta)*q(1))/q;
kappaL*h^(phi)=w;        
k=(1-delta)*k(-1)+(1-kappaI/2*(i/i(-1)-1)^2)*i;
1=q*(1-kappaI/2*(i/i(-1)-1)^2-kappaI*(i/i(-1)-1)*i/i(-1))+kappaI*beta*lambda(1)/lambda*q(1)*(i(1)/i-1)*(i(1)/i)^2;    
% Firms
yT=aT*kT^alphaT*hT^(1-alphaT);
yN=aN*kN^alphaN*hN^(1-alphaN);
(1-alphaT)*yT=w*hT*p;
(1-alphaN)*yN=w*hN*p/pN;
alphaT*yT=rk*kT*p;
alphaN*yN=rk*kN*p/pN;
% Market clearing
yN=(1-gamma)*(pN/p)^(-eta)*(c+i+g-yO*tO);
k(-1)=kT+kN;
h=hT+hN;
gdp=c+i+g-yO*tO+tb
;
//Mining output and price
yO=yOss*(yO(-1)/yOss)^rhoyO*exp(sdyO*vyO);
pO=pO(-1)^rhopO*exp(sdpO*vpO);
//
% Prices
p^(1-eta)=(1-gamma)*pN^(1-eta)+gamma;
% Shocks
log(aT)=(1-rhoaT)*log(aTss)+rhoaT*log(aT(-1))+sdT*vaT;
log(aN)=rhoaN*log(aN(-1))+sdN*vaN;  
log(g-yO*tO)=(1-rhog)*log(gss-yOss*tO)+rhog*log(g(-1)-yO(-1)*tO)+sdg*vg;  
rz=(1-rhop)*1/beta+rhop*rz(-1)+sdp*vp;
log(ac)=rhoc*log(ac(-1))+sdc*vc;
% Auxiliary variables
gdp=(pN*yN+yT+pO*yO)/p;
tb=-d/p+rz(-1)*d(-1)/p+kappaD/(2*p)*(d-dss)^2;
// check
Z=yT-(gamma*(p)^(eta)*(c+i+g-yO*tO)+p*tb); 
dy=d/(p*gdp);
% Log variables
clog=log(c); 
wlog=log(w); 
hlog=log(h);  
hNlog=log(hN); 
hTlog=log(hT);
klog=log(k); 
kNlog=log(kN); 
kTlog=log(kT);
ilog=log(i); 
yTlog=log(yT);
yNlog=log(yN); 
gdplog=log(gdp);
plog=log(p); 
pNlog=log(pN);
dlog=log(d);
//// observation equation
NX_GDP=tb/gdp;
GDP_Growth=gdp/gdp(-1)-1;
G_Growth=g/g(-1)-1;
M_GDP_Growth=yO/yO(-1)-1;
M_Price_Growth=pO/pO(-1)-1;
Cons_Growth=c/c(-1)-1;
Foreign_Rate=rz-1;

end;

%% Steady State values are estimated in seperate file, console_mining.mat

initval;
aT=aTss;
kT=kTss;
kN=kNss;
pN=pNss;
hN=hNss;
// mining steady state
pO=1;
yO=yOss;
// steady state preference shock
ac=1;
// 
h=1/3;
rz=1/beta;
q=1;
rk=1/beta-(1-delta); 
p=pss;

k=kTss+kNss;
i=delta*k;
hT=hTss;
yT=aT*kT^alphaT*hT^(1-alphaT);
yN=kN^alphaN*hN^(1-alphaN);
gdp=(pN*yN+yT)/(p-O);
d=dss;
tb=d/p*(1/beta-1);
g=gss;
c=gdp-i-g+yO*tO-tb;
w=(1-alphaT)*yT/(hT*p);
lambda=(c-kappaL/(1+phi)*h^(1+phi))^-sigma; 
aN=1;
clog=log(c);  
wlog=log(w); 
hlog=log(h);  
hNlog=log(hN); 
hTlog=log(hT);
klog=log(k); 
kNlog=log(kN); 
kTlog=log(kT);
ilog=log(i); 
yTlog=log(yT);
yNlog=log(yN); 
gdplog=log(gdp);
plog=log(p); 
pNlog=log(pN);
dy=d/(p*gdp);
dlog=log(d);
Z=0;
end;

steady;
check;

%% Shocks scaled unit standard deviation
shocks;

var vaT;
periods 1:4;
values -1;
 
var vaN;
periods 1:4;
values -1;
 
var vc;
periods 1:4;
values -1;

var vg;
periods 1:8;
values 1;

var vp;
periods 1:4;
values -1;

var vyO;
periods 1:4;
values -1;

var vpO;
periods 1:4;
values -1; 
 
end; 

simul(periods=2000);


